﻿#include "pch.h"
#include "ClipToTextMain.h"

#include <DirectXColors.h>			// For named colors	
#include "Common\DirectXHelper.h"	// For ThrowIfFailed

using namespace ClipToText;

// Loads and initializes application assets when the application is loaded.
ClipToTextMain::ClipToTextMain(const std::shared_ptr<DeviceResources>& deviceResources) :
	m_deviceResources(deviceResources)
{

	m_clipToTextRenderer = std::unique_ptr<ClipToTextRenderer>(new ClipToTextRenderer(m_deviceResources));
    CreateWindowSizeDependentResources();

	// TODO: Change timer settings if you want something other than the default variable timestep mode.
	// eg. for 60 FPS fixed timestep update logic, call:
	/*
	m_timer.SetFixedTimeStep(true);
	m_timer.SetTargetElapsedSeconds(1.0 / 60);
	*/
	m_deviceResources->RegisterDeviceNotify(this);
}

ClipToTextMain::~ClipToTextMain()
{
	// Deregister from device notification
	m_deviceResources->RegisterDeviceNotify(nullptr);
}

// Updates application state when the window size changes (e.g. device orientation change)
void ClipToTextMain::CreateWindowSizeDependentResources() 
{
    m_clipToTextRenderer->CreateWindowSizeDependentResources();
}

// Updates the application state once per frame.
void ClipToTextMain::Update() 
{
	// Update scene objects.
	m_timer.Tick([&]()
	{
		m_clipToTextRenderer->Update(m_timer);
	});
}

// Renders the current frame according to the current application state.
// Returns true if the frame was rendered and is ready to be displayed.
bool ClipToTextMain::Render() 
{
	// Don't try to render anything before the first Update.
	if (m_timer.GetFrameCount() == 0)
	{
		return false;
	}

	return m_clipToTextRenderer->Render();
}

// Notifies renderers that device resources need to be released.
void ClipToTextMain::OnDeviceLost()
{
	m_clipToTextRenderer->ReleaseDeviceDependentResources();
}

// Notifies renderers that device resources may now be recreated.
void ClipToTextMain::OnDeviceRecreated()
{
	m_clipToTextRenderer->CreateDeviceDependentResources();
	CreateWindowSizeDependentResources();
}
