﻿#include "pch.h"
#include "OscillatingTextMain.h"

#include <DirectXColors.h>			// For named colors	
#include "Common\DirectXHelper.h"	// For ThrowIfFailed

using namespace OscillatingText;

// Loads and initializes application assets when the application is loaded.
OscillatingTextMain::OscillatingTextMain(const std::shared_ptr<DeviceResources>& deviceResources) :
	m_deviceResources(deviceResources)
{
	m_oscillatingTextRenderer = std::unique_ptr<OscillatingTextRenderer>(new OscillatingTextRenderer(m_deviceResources));
    CreateWindowSizeDependentResources();

	// TODO: Change timer settings if you want something other than the default variable timestep mode.
	// eg. for 60 FPS fixed timestep update logic, call:
	/*
	m_timer.SetFixedTimeStep(true);
	m_timer.SetTargetElapsedSeconds(1.0 / 60);
	*/
	m_deviceResources->RegisterDeviceNotify(this);
}

OscillatingTextMain::~OscillatingTextMain()
{
	// Deregister from device notification
	m_deviceResources->RegisterDeviceNotify(nullptr);
}

// Updates application state when the window size changes (e.g. device orientation change)
void OscillatingTextMain::CreateWindowSizeDependentResources() 
{
	m_oscillatingTextRenderer->CreateWindowSizeDependentResources();
}

// Updates the application state once per frame.
void OscillatingTextMain::Update() 
{
	// Update scene objects.
	m_timer.Tick([&]()
	{
		m_oscillatingTextRenderer->Update(m_timer);
	});
}

// Renders the current frame according to the current application state.
// Returns true if the frame was rendered and is ready to be displayed.
bool OscillatingTextMain::Render() 
{
	// Don't try to render anything before the first Update.
	if (m_timer.GetFrameCount() == 0)
	{
		return false;
	}

	m_oscillatingTextRenderer->Render();

	return true;
}

// Notifies renderers that device resources need to be released.
void OscillatingTextMain::OnDeviceLost()
{
	m_oscillatingTextRenderer->ReleaseDeviceDependentResources();
}

// Notifies renderers that device resources may now be recreated.
void OscillatingTextMain::OnDeviceRecreated()
{
	m_oscillatingTextRenderer->CreateDeviceDependentResources();
	CreateWindowSizeDependentResources();
}
