#include "pch.h"
#include "InterrogableGeometrySink.h"

// IUnknown methods
ULONG STDMETHODCALLTYPE InterrogableGeometrySink::AddRef()
{
    m_refCount++;
    return m_refCount;
}

ULONG STDMETHODCALLTYPE InterrogableGeometrySink::Release()
{
    m_refCount--;
    LONG newCount = m_refCount;

    if (m_refCount == 0)
        delete this;

    return newCount;
}

HRESULT STDMETHODCALLTYPE InterrogableGeometrySink::QueryInterface(_In_ REFIID riid, _Outptr_ void** ppOutput)
{
    *ppOutput = nullptr;
    HRESULT hr = S_OK;

    if (riid == __uuidof(ID2D1SimplifiedGeometrySink))
    {
        *ppOutput = static_cast<ID2D1SimplifiedGeometrySink*>(this);
        AddRef();
    }
    else if (riid == __uuidof(IUnknown))
    {
        *ppOutput = this;
        AddRef();
    }
    else
    {
        hr = E_NOINTERFACE;
    }
    return hr;
}

// ID2D1SimplifiedGeometrySink methods
InterrogableGeometrySink::InterrogableGeometrySink() : m_refCount(0)
{
}

void InterrogableGeometrySink::SetFillMode(D2D1_FILL_MODE fillMode)
{
    m_pathGeometryData.FillMode = fillMode;
}

void InterrogableGeometrySink::SetSegmentFlags(D2D1_PATH_SEGMENT vertexFlags)
{
    // Ignore
}

void InterrogableGeometrySink::BeginFigure(D2D1_POINT_2F startPoint, 
                                           D2D1_FIGURE_BEGIN figureBegin)
{
    m_pathFigureData.StartPoint = startPoint;
    m_pathFigureData.FigureBegin = figureBegin;
    m_pathFigureData.Segments.clear();
}

void InterrogableGeometrySink::AddLines(const D2D1_POINT_2F *points, 
                                        UINT pointsCount)
{
    PathSegmentData polyLineSegment;
    polyLineSegment.IsBezier = false;
    polyLineSegment.Points.assign(points, points + pointsCount);
    m_pathFigureData.Segments.push_back(polyLineSegment);
}

void InterrogableGeometrySink::AddBeziers(const D2D1_BEZIER_SEGMENT *beziers, 
                                          UINT beziersCount)
{
    PathSegmentData polyBezierSegment;
    polyBezierSegment.IsBezier = true;
    polyBezierSegment.Beziers.assign(beziers, beziers + beziersCount);
    m_pathFigureData.Segments.push_back(polyBezierSegment);
}

void InterrogableGeometrySink::EndFigure(D2D1_FIGURE_END figureEnd)
{
    m_pathFigureData.FigureEnd = figureEnd;
    m_pathGeometryData.Figures.push_back(m_pathFigureData);
}

HRESULT InterrogableGeometrySink::Close()
{
    // Assume that the class accessing the geometry sink knows what it's doing
    return S_OK;
}

// Method for this implementation
PathGeometryData InterrogableGeometrySink::GetPathGeometryData()
{
    return m_pathGeometryData;
}
