#include "pch.h"
#include "PathGeometryDataEtc.h"
#include "..\Common\DirectXHelper.h"

Microsoft::WRL::ComPtr<ID2D1PathGeometry> PathGeometryData::GeneratePathGeometry(ID2D1Factory * factory)
{
    Microsoft::WRL::ComPtr<ID2D1PathGeometry> pathGeometry;

    DX::ThrowIfFailed(
        factory->CreatePathGeometry(&pathGeometry)
        );

    Microsoft::WRL::ComPtr<ID2D1GeometrySink> geometrySink;

    DX::ThrowIfFailed(
        pathGeometry->Open(&geometrySink)
        );

    geometrySink->SetFillMode(this->FillMode);

    for (const PathFigureData& pathFigure : Figures)
    {
        geometrySink->BeginFigure(pathFigure.StartPoint, pathFigure.FigureBegin);

        for (const PathSegmentData& pathSegment : pathFigure.Segments)
        {
            if (pathSegment.IsBezier)
            {
                geometrySink->AddBeziers(pathSegment.Beziers.data(), pathSegment.Beziers.size());
            }
            else
            {
                geometrySink->AddLines(pathSegment.Points.data(), pathSegment.Points.size());
            }
        }

        geometrySink->EndFigure(pathFigure.FigureEnd);
    }

    geometrySink->Close();

    return pathGeometry;
}
