﻿#pragma once

#include <string>
#include "..\DeviceResources.h"
#include "..\Common\StepTimer.h"
#include "PathGeometryDataEtc.h"

namespace RealTextEditor
{
	class RealTextEditorRenderer
	{
	public:
		RealTextEditorRenderer(const std::shared_ptr<DeviceResources>& deviceResources);
		void CreateDeviceDependentResources();
		void ReleaseDeviceDependentResources();
        void CreateWindowSizeDependentResources();
		void Update(DX::StepTimer const& timer);
		bool Render();

        // Asynchronous pointer events
        void OnPointerPressed(unsigned int id, D2D1_POINT_2F pointerPos);
        void OnPointerMoved(unsigned int id, D2D1_POINT_2F pointerPos);
        void OnPointerReleased(unsigned int id, D2D1_POINT_2F pointerPos);

	private:
        static const int m_dotRadius = 8;

		// Cached pointer to device resources.
		std::shared_ptr<DeviceResources> m_deviceResources;

        bool m_needsRedraw;
        bool m_editMode;
        D2D1::Matrix3x2F m_displayTransform;
        D2D1::Matrix3x2F m_inverseDisplayTransform;

        PathGeometryData                          m_pathGeometryData;
        Microsoft::WRL::ComPtr<ID2D1PathGeometry> m_textGeometry;
        D2D1_RECT_F                               m_geometryBounds;

        Microsoft::WRL::ComPtr<ID2D1SolidColorBrush>    m_blackBrush;
		Microsoft::WRL::ComPtr<ID2D1SolidColorBrush>    m_darkBlueBrush;
        Microsoft::WRL::ComPtr<ID2D1SolidColorBrush>    m_redBrush;
        Microsoft::WRL::ComPtr<ID2D1SolidColorBrush>    m_greenBrush;
		Microsoft::WRL::ComPtr<ID2D1DrawingStateBlock>  m_stateBlock;

        // Pointer tracking
        struct PointerInfo
        {
            size_t FigureIndex;
            size_t SegmentIndex;
            bool   IsBezier;
            size_t BezierIndex;
            size_t PointIndex;
            D2D1_POINT_2F LastPointerPosition;
        };

        std::map<unsigned int, PointerInfo> pointerTracker;
        void TranslatePoint(PointerInfo& pointerInfo, D2D1_POINT_2F pointerPos);

        inline bool HitTest(D2D1_POINT_2F pt1, D2D1_POINT_2F pt2, float offset)
        {
            return std::abs(pt1.x - pt2.x) < offset && std::abs(pt1.y - pt2.y) < offset;
        }
	};
}