﻿#include "pch.h"
#include "RealTextEditorMain.h"

#include <DirectXColors.h>			// For named colors	
#include "Common\DirectXHelper.h"	// For ThrowIfFailed

using namespace RealTextEditor;

using namespace D2D1;                   // for Point2F
using namespace Windows::Foundation;    // for Point
using namespace Windows::UI::Core;      // for PointerEventArgs

// Loads and initializes application assets when the application is loaded.
RealTextEditorMain::RealTextEditorMain(const std::shared_ptr<DeviceResources>& deviceResources) :
	m_deviceResources(deviceResources)
{
	m_realTextEditorRenderer = std::unique_ptr<RealTextEditorRenderer>(new RealTextEditorRenderer(m_deviceResources));
    CreateWindowSizeDependentResources();

	// TODO: Change timer settings if you want something other than the default variable timestep mode.
	// eg. for 60 FPS fixed timestep update logic, call:
	/*
	m_timer.SetFixedTimeStep(true);
	m_timer.SetTargetElapsedSeconds(1.0 / 60);
	*/
	m_deviceResources->RegisterDeviceNotify(this);
}

RealTextEditorMain::~RealTextEditorMain()
{
	// Deregister from device notification
	m_deviceResources->RegisterDeviceNotify(nullptr);
}

// Updates application state when the window size changes (e.g. device orientation change)
void RealTextEditorMain::CreateWindowSizeDependentResources() 
{
	m_realTextEditorRenderer->CreateWindowSizeDependentResources();
}

// Updates the application state once per frame.
void RealTextEditorMain::Update() 
{
	// Update scene objects.
	m_timer.Tick([&]()
	{
		m_realTextEditorRenderer->Update(m_timer);
	});
}

// Renders the current frame according to the current application state.
// Returns true if the frame was rendered and is ready to be displayed.
bool RealTextEditorMain::Render() 
{
	// Don't try to render anything before the first Update.
	if (m_timer.GetFrameCount() == 0)
	{
		return false;
	}

	return m_realTextEditorRenderer->Render();
}

// Notifies renderers that device resources need to be released.
void RealTextEditorMain::OnDeviceLost()
{
	m_realTextEditorRenderer->ReleaseDeviceDependentResources();
}

// Notifies renderers that device resources may now be recreated.
void RealTextEditorMain::OnDeviceRecreated()
{
	m_realTextEditorRenderer->CreateDeviceDependentResources();
	CreateWindowSizeDependentResources();
}

void RealTextEditorMain::OnPointerPressed(unsigned int id, Point point)
{
    m_realTextEditorRenderer->OnPointerPressed(id, Point2F(point.X, point.Y));
}

void RealTextEditorMain::OnPointerMoved(unsigned int id, Point point)
{
    m_realTextEditorRenderer->OnPointerMoved(id, Point2F(point.X, point.Y));
}

void RealTextEditorMain::OnPointerReleased(unsigned int id, Point point)
{
    m_realTextEditorRenderer->OnPointerReleased(id, Point2F(point.X, point.Y));
}

