﻿using SimpleDWriteLib;
using System;
using Windows.Foundation;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Media;

namespace RippleText
{
    public sealed partial class MainPage : Page
    {
        TextGeometryGenerator txtGeoGenerator = new TextGeometryGenerator();
        PathGeometry originalGeometry;
        PathGeometry transformedGeometry;
        Rect geometryBounds;

        public MainPage()
        {
            this.InitializeComponent();

            txtGeoGenerator.Text = "Ripple Text";
            txtGeoGenerator.FontSize = 192;
            txtGeoGenerator.Interpolate = true;

            lstbox.ItemsSource = txtGeoGenerator.FontFamilies;
            lstbox.SelectedItem = "Times New Roman";

            CompositionTarget.Rendering += OnCompositionTargetRendering;
        }

        void OnListBoxSelectionChanged(object sender, SelectionChangedEventArgs args)
        {
            if (lstbox.SelectedItem == null)
            {
                originalGeometry = null;
                transformedGeometry = null;
                path.Data = null;
            }
            else
            {
                txtGeoGenerator.FontFamily = (string)lstbox.SelectedItem;
                originalGeometry = txtGeoGenerator.FlattenedGeometry;
                geometryBounds = originalGeometry.Bounds;

                // Force TextGeometryGenerator to generate a new geometry
                string text = txtGeoGenerator.Text;
                txtGeoGenerator.Text = "";
                txtGeoGenerator.Text = text;

                // Save it and set to to the Path
                transformedGeometry = txtGeoGenerator.FlattenedGeometry;
                path.Data = transformedGeometry;
            }
        }

        void OnCompositionTargetRendering(object sender, object arg)
        {
            if (originalGeometry == null || transformedGeometry == null)
                return;

            RenderingEventArgs renderingArgs = arg as RenderingEventArgs;
            double t = renderingArgs.RenderingTime.TotalSeconds / 10;

            for (int figIndex = 0; figIndex < originalGeometry.Figures.Count; figIndex++)
            {
                PathFigure srcPathFigure = originalGeometry.Figures[figIndex];
                PathFigure dstPathFigure = transformedGeometry.Figures[figIndex];

                for (int segIndex = 0; segIndex < srcPathFigure.Segments.Count; segIndex++)
                {
                    PointCollection srcPoints = null;
                    PointCollection dstPoints = null;

                    if (srcPathFigure.Segments[segIndex] is PolyLineSegment)
                    {
                        srcPoints = (srcPathFigure.Segments[segIndex] as PolyLineSegment).Points;
                        dstPoints = (dstPathFigure.Segments[segIndex] as PolyLineSegment).Points;
                    }
                    else
                    {
                        srcPoints = (srcPathFigure.Segments[segIndex] as PolyBezierSegment).Points;
                        dstPoints = (dstPathFigure.Segments[segIndex] as PolyBezierSegment).Points;
                    }

                    for (int ptIndex = 0; ptIndex < srcPoints.Count; ptIndex++)
                    {
                        Point srcPt = srcPoints[ptIndex];

                        double horz = (srcPt.X - geometryBounds.Left) / geometryBounds.Width;
                        double vert = (srcPt.Y - geometryBounds.Top) / geometryBounds.Height * 2 - 1;

                        // Alternatively, set vert to 1 (or other constant) for ripple that doesn't change character height
                        // vert = 1;

                        Point dstPt = new Point(srcPt.X, srcPt.Y + 30 * vert * Math.Sin((t + horz) * 10 * Math.PI));
                        dstPoints[ptIndex] = dstPt;
                    }
                }
            }
        }
    }
}
