#include "pch.h"
#include "TextGeometryGenerator.h"
#include "InteroperableGeometrySink.h"

using namespace SimpleDWriteLib;

using namespace Microsoft::WRL;
using namespace Platform;
using namespace Platform::Collections;
using namespace Windows::Foundation::Collections;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Interop;
using namespace Windows::UI::Xaml::Media;

// Static DependencyProperty objects
DependencyProperty^ TextGeometryGenerator::fontFamiliesProperty =
    DependencyProperty::Register("FontFamilies",
                                 TypeName(IVector<String^>::typeid),
                                 TypeName(TextGeometryGenerator::typeid),
                                 ref new PropertyMetadata(ref new Vector<String^>));

DependencyProperty^ TextGeometryGenerator::textProperty =
    DependencyProperty::Register("Text",
                                 TypeName(String::typeid),
                                 TypeName(TextGeometryGenerator::typeid),
                                  ref new PropertyMetadata("", ref new PropertyChangedCallback(OnPropertyChangedStatic)));

DependencyProperty^ TextGeometryGenerator::fontFamilyProperty =
    DependencyProperty::Register("FontFamily",
                                 TypeName(String::typeid),
                                 TypeName(TextGeometryGenerator::typeid),
                                 ref new PropertyMetadata("", ref new PropertyChangedCallback(OnPropertyChangedStatic)));

DependencyProperty^ TextGeometryGenerator::fontSizeProperty =
    DependencyProperty::Register("FontSize",
                                 TypeName(double::typeid),
                                 TypeName(TextGeometryGenerator::typeid),
                                 ref new PropertyMetadata(12.0, ref new PropertyChangedCallback(OnPropertyChangedStatic)));

DependencyProperty^ TextGeometryGenerator::fontStyleProperty =
    DependencyProperty::Register("FontStyle",
                                 TypeName(Windows::UI::Text::FontStyle::typeid),
                                 TypeName(TextGeometryGenerator::typeid),
                                 ref new PropertyMetadata(Windows::UI::Text::FontStyle::Normal, 
                                                          ref new PropertyChangedCallback(OnPropertyChangedStatic)));

DependencyProperty^ TextGeometryGenerator::fontWeightProperty =
    DependencyProperty::Register("FontWeight",
                                 TypeName(Windows::UI::Text::FontWeight::typeid),
                                 TypeName(TextGeometryGenerator::typeid),
                                 ref new PropertyMetadata(Windows::UI::Text::FontWeights::Normal, 
                                                          ref new PropertyChangedCallback(OnPropertyChangedStatic)));

DependencyProperty^ TextGeometryGenerator::interpolateProperty =
    DependencyProperty::Register("Interpolate",
                                 TypeName(bool::typeid),
                                 TypeName(TextGeometryGenerator::typeid),
                                 ref new PropertyMetadata(false, ref new PropertyChangedCallback(OnPropertyChangedStatic)));

DependencyProperty^ TextGeometryGenerator::geometryProperty =
    DependencyProperty::Register("Geometry", 
                                 TypeName(PathGeometry::typeid),
                                 TypeName(TextGeometryGenerator::typeid),
                                 ref new PropertyMetadata(nullptr));

DependencyProperty^ TextGeometryGenerator::flattenedGeometryProperty =
    DependencyProperty::Register("FlattenedGeometry",
                                 TypeName(PathGeometry::typeid),
                                 TypeName(TextGeometryGenerator::typeid),
                                 ref new PropertyMetadata(nullptr));

TextGeometryGenerator::TextGeometryGenerator()
{
    // Create Direct2D factory
    D2D1_FACTORY_OPTIONS options;
    ZeroMemory(&options, sizeof(D2D1_FACTORY_OPTIONS));

#if defined(_DEBUG)
    // If the project is in a debug build, enable Direct2D debugging via SDK Layers.
    options.debugLevel = D2D1_DEBUG_LEVEL_INFORMATION;
#endif

    HRESULT hr = D2D1CreateFactory(D2D1_FACTORY_TYPE_SINGLE_THREADED, __uuidof(ID2D1Factory), &options, &m_d2dFactory);

    if (FAILED(hr))
        throw ref new COMException(hr, "D2D1CreateFactory failure");

    // Create DirectWrite factory
    hr = DWriteCreateFactory(DWRITE_FACTORY_TYPE_SHARED, _uuidof(m_dwriteFactory), &m_dwriteFactory);

    if (FAILED(hr))
        throw ref new COMException(hr, "DWriteCreateFactory failure");

    // Get system font collection and font families
    hr = m_dwriteFactory->GetSystemFontCollection(&m_fontCollection, true);

    if (FAILED(hr))
        throw ref new COMException(hr, "GetSystemFontCollection failure");

    unsigned int count = m_fontCollection->GetFontFamilyCount();
    std::vector<String^> fonts;

    // Loop through the font families
    for (unsigned int i = 0; i < count; i++)
    {
        ComPtr<IDWriteFontFamily> fontFamily;
        hr = m_fontCollection->GetFontFamily(i, &fontFamily);

        if (FAILED(hr))
            throw ref new COMException(hr, "GetFontFamily failure");

        ComPtr<IDWriteLocalizedStrings> familyNames;
        hr = fontFamily->GetFamilyNames(&familyNames);

        if (FAILED(hr))
            throw ref new COMException(hr, "GetFamilyNames failure");

        unsigned int index = 0;
        BOOL exists;
        hr = familyNames->FindLocaleName(L"en-us", &index, &exists);

        if (FAILED(hr))
            throw ref new COMException(hr, "FindLocaleName failure");

        if (exists)
        {
            unsigned int length = 0;
            hr = familyNames->GetStringLength(index, &length);

            if (FAILED(hr))
                throw ref new COMException(hr, "GetStringLength failure");

            std::vector<wchar_t> str(length + 1);
            hr = familyNames->GetString(index, str.data(), length + 1);

            if (FAILED(hr))
                throw ref new COMException(hr, "GetString failure");

            // Add the font family name to the collection
            fonts.push_back(ref new String(str.data()));
        }
    }

    // Sort the font family names
    std::sort(fonts.begin(), fonts.end(), [](String^ str1, String^ str2)
    {
        return str1 < str2;
    });

    // Set the FontFamiles property
    SetValue(fontFamiliesProperty, ref new Vector<String^>(fonts.data(), fonts.size()));
}

void TextGeometryGenerator::OnPropertyChangedStatic(DependencyObject^ obj, DependencyPropertyChangedEventArgs^ args)
{
    safe_cast<TextGeometryGenerator^>(obj)->OnPropertyChanged(args);
}

void TextGeometryGenerator::OnPropertyChanged(DependencyPropertyChangedEventArgs^ args)
{
    if (this->Text == nullptr || this->Text->Length() == 0 ||
        this->FontFamily == nullptr || this->FontFamily->Length() == 0)
    {
        SetValue(geometryProperty, nullptr);
        return;
    }

    // Find the family name
    unsigned int index;
    BOOL exists;
    HRESULT hr = m_fontCollection->FindFamilyName(this->FontFamily->Data(), &index, &exists);

    if (FAILED(hr))
        throw ref new COMException(hr, "FindFamilyName failure");

    // If family doesn't exist, just set the Geometry property to null
    if (!exists)
    {
        SetValue(geometryProperty, nullptr);
        return;
    }

    // Get the font family
    ComPtr<IDWriteFontFamily> fontFamily;
    hr = m_fontCollection->GetFontFamily(index, &fontFamily);

    if (FAILED(hr))
        throw ref new COMException(hr, "GetFontFamily failure");

    // Get the font
    ComPtr<IDWriteFont> font;
    hr = fontFamily->GetFirstMatchingFont(
            static_cast<DWRITE_FONT_WEIGHT>(this->FontWeight.Weight),
            DWRITE_FONT_STRETCH::DWRITE_FONT_STRETCH_NORMAL,
            static_cast<DWRITE_FONT_STYLE>(this->FontStyle),
            &font);

    if (FAILED(hr))
        throw ref new COMException(hr, "GetFirstMatchingFont failure");

    // Create the font face
    ComPtr<IDWriteFontFace> fontFace;
    hr = font->CreateFontFace(&fontFace);

    if (FAILED(hr))
        throw ref new COMException(hr, "CreateFontFace failure");

    // Convert text to glyph indices
    std::wstring text = this->Text->Data();
    uint32 glyphCount = text.length();
    std::vector<uint32> codePoints(glyphCount);

    for (uint32 i = 0; i < glyphCount; i++)
        codePoints[i] = text[i];

    std::vector<uint16> glyphIndices = std::vector<uint16>(glyphCount);
    hr = fontFace->GetGlyphIndices(codePoints.data(), glyphCount, glyphIndices.data());

    if (FAILED(hr))
        throw ref new COMException(hr, "GetGlyphIndices failure");

    // Attempt to set glyph offsets so geometry begins at y coordinates of ~0
    DWRITE_FONT_METRICS fontFaceMetrics;
    fontFace->GetMetrics(&fontFaceMetrics);
    std::vector<DWRITE_GLYPH_OFFSET> glyphOffsets = std::vector<DWRITE_GLYPH_OFFSET>(glyphCount);
    float normalizer = -float(this->FontSize) * fontFaceMetrics.capHeight / fontFaceMetrics.designUnitsPerEm;

    for (uint32 i = 0; i < glyphCount; i++)
    {
        DWRITE_GLYPH_OFFSET offset;
        offset.advanceOffset = 0;
        offset.ascenderOffset = normalizer;
        glyphOffsets[i] = offset;
    }

    ComPtr<InteroperableGeometrySink> interoperableGeometrySink = new InteroperableGeometrySink(this->Interpolate);

    hr = fontFace->GetGlyphRunOutline(float(this->FontSize), 
                                      glyphIndices.data(), 
                                      nullptr, 
                                      glyphOffsets.data(), 
                                      glyphCount, 
                                      false, 
                                      false, 
                                      interoperableGeometrySink.Get());

    if (FAILED(hr))
        throw ref new COMException(hr, "GetGlyphRunOutline failure");

    // Set the Geometry property to the path geometry from the InteroperableGeometrySink
    SetValue(TextGeometryGenerator::GeometryProperty, interoperableGeometrySink->GetPathGeometry());

    ComPtr<ID2D1PathGeometry> dxPathGeometry;
    hr = m_d2dFactory->CreatePathGeometry(&dxPathGeometry);

    if (FAILED(hr))
        throw ref new COMException(hr, "CreatePathGeometry failure");

    ComPtr<ID2D1GeometrySink> dxGeometrySink;

    hr = dxPathGeometry->Open(&dxGeometrySink);

    if (FAILED(hr))
        throw ref new COMException(hr, "CreatePathGeometry failure");

    hr = fontFace->GetGlyphRunOutline(float(this->FontSize),
                                      glyphIndices.data(),
                                      nullptr,
                                      glyphOffsets.data(),
                                      glyphCount,
                                      false,
                                      false,
                                      dxGeometrySink.Get());

    if (FAILED(hr))
        throw ref new COMException(hr, "GetGlyphRunOutline failure");

    hr = dxGeometrySink->Close();

    if (FAILED(hr))
        throw ref new COMException(hr, "geometrySink->Close failure");

    // Recreate InteroperableGeometrySink
    interoperableGeometrySink = new InteroperableGeometrySink(this->Interpolate);
    hr = dxPathGeometry->Simplify(D2D1_GEOMETRY_SIMPLIFICATION_OPTION_LINES, 
                                  D2D1::IdentityMatrix(), 
                                  interoperableGeometrySink.Get());

    if (FAILED(hr))
        throw ref new COMException(hr, "Simplify failure");

    SetValue(TextGeometryGenerator::FlattenedGeometryProperty, interoperableGeometrySink->GetPathGeometry());
}

